﻿##------------------------------------------------------------------------------------------------
##
##  Create-MailboxFolderPermissionReport.ps1
##
##   Version 1.0.5
##
##
##   Copyright (c) 2016 Martin Mueller - www.sh-soft.com
##   
##   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
##   and associated documentation files (the "Software"), to deal in the Software without 
##   restriction, including without limitation the rights to use, copy, modify, merge, publish, 
##   distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
##   Software is furnished to do so, subject to the following conditions:
##   
##   The above copyright notice and this permission notice shall be included in all copies or 
##   substantial portions of the Software.
##   
##   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
##   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
##   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
##   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
##   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##   (The MIT License (MIT))
##
##------------------------------------------------------------------------------------------------

<#
.SYNOPSIS
Creates a tree view report for permissions on individual mailbox folders.



.DESCRIPTION

This script creates a html based report. Each folder within a given mailbox gets displayed within a tree view.

If there a permissions set to this specific folder they get displayed below the folder name.

The user needs at least "Exchange View Only" admin rights inside the exchange organisation!

for mor information please go to www.sh-soft.com/en/scripting/ps/ps-create-mailboxfolderpermissionreport.html

HINT: you cannot use this script directly in pipeline with the Get-Mailbox commandlet! See Examples for a workaround...



.PARAMETER ExchangeServer 

[required] the fqdn name or IP address of your exchange server. The script uses this server for remote shell.



.PARAMETER MailboxName

[required] this one can be Alias, Distinguished name (DN), GUID, Name, Display name, LegacyExchangeDN or Email address



.PARAMETER ReportName

can be used to specify a report file name otherwise the default name will be used


.PARAMETER HideDefaultPermissions

can be used if the default permissions should be displayed then set it to $false


.PARAMETER HideMailboxOwner

can be used if the mailbox owners permissions should be display then set it to $false


.PARAMETER AppendReport

If set to $true the report will append to an existing file (this is useful if you want to display all reports in one file when using the script in pipe


.EXAMPLE

Create a report for the user with the SAMAccountName "John Oliva" while connecting to exchange server 

"msx.yourdomain.com" in your logged on users %userhome% drive:

.\Create-MailboxFolderPermissionReport.ps1 -ExchangeServer msx.yourdomain.com -MailboxName "John Oliva"



.EXAMPLE

Create a report for the user with the SAMAccountName "John Oliva" while connecting to exchange server 

"msx.yourdomain.com" to the file: "C:\reports\JohnOliva.html"

.\Create-MailboxFolderPermissionReport.ps1 -ExchangeServer msx.yourdomain.com -MailboxName "John Oliva" -ReportName "C:\reports\JohnOliva.html"


.EXAMPLE

Create a report for all mailboxes within the Exchange organization while connecting to exchange server

"msx.yourdomain.com" to the file: "C:\reports\AllPermissions.html"

$AllMailboxes = Get-Mailbox -resultsize unlimited; $AllMailboxes | .\Create-MailboxFolderPermissionReport.ps1 -ExchangeServer msx.yourdomain.com -ReportName "C:\reports\AllPermissions.html" -AppendReport $true


.EXAMPLE

Create a report for all ADDS users in a given organizational unit (OU) while connecting to exchange server

"msx.yourdomain.com" to the file: "C:\reports\AllPermissions.html"

HINT: Users without exchange mailboxes will be displayed as ERROR. This is normal because no mailbox attributes can be used

Import-Mobule ActiveDirectory; Get-ADUser -SearchBase "ou=MyRockBand,dc=yourdomain,dc=com" -Filter * | .\Create-MailboxFolderPermissionReport.ps1 -ExchangeServer msx.yourdomain.com -ReportName "C:\reports\AllPermissions.html" -AppendReport $true

#>

#------------------------------------------------------------------------------------------------
# Parameter block
#------------------------------------------------------------------------------------------------
param ( 
	[Parameter(Mandatory=$true,HelpMessage='Exchange server to connect to',Position=0)]
	[string]$ExchangeServer,
	[Parameter(Mandatory=$true,HelpMessage='Mailbox for which to create the report',valueFromPipeline=$true)]
	[string]$MailboxName,
	[Parameter(HelpMessage='Report filename')]
	[string]$ReportName = $null,
	[Parameter(HelpMessage='You can also enable default folder permissions to be displayed')]
	[bool]$HideDefaultPermissions = $true,
	[Parameter(HelpMessage='You can also enable the mailbox owner permission to be displayed')]
	[bool]$HideMailboxOwner = $true,
	[Parameter(HelpMessage='Append to existing file')]
	[bool]$AppendReport = $false

)
BEGIN {
	#------------------------------------------------------------------------------------------------
	#  Connect / disconnect to the remote exchange shell for exchange command-lets...
	#------------------------------------------------------------------------------------------------
	$global:MyPSSession = $null
	function Handle-ExchangeShell ($ExchangeServerName, $Mode) {
		if ($Mode -eq "connect") {
			Write-Host -ForegroundColor Green ("connecting to remote exchange shell...")
			$Session = New-PSSession -ConfigurationName Microsoft.Exchange `
										-ConnectionUri http://$ExchangeServerName/PowerShell/ `
										-Authentication Kerberos
			$global:MyPSSession = $Session.Id
			$null = Import-PSSession $Session -ErrorAction SilentlyContinue -WarningAction SilentlyContinue -DisableNameChecking -OutVariable $null
		}
		elseif ($Mode -eq "disconnect" -and $global:MyPSSession -ne $null) {
			Write-Host -ForegroundColor Green ("disconnecting from remote exchange shell...")
			Remove-PSSession -Id $global:MyPSSession
		}
	}

	#------------------------------------------------------------------------------------------------
	# Connecting to the exchange shell
	#------------------------------------------------------------------------------------------------

	$Exchange
	if ((Get-Command -Name Get-Mailbox -ErrorAction SilentlyContinue).Length -eq 0 ) {
			Handle-ExchangeShell -ExchangeServerName $ExchangeServer -Mode "connect"
	}
}	

##------------------------------------------------------------------------------------------------
##  main block...
##------------------------------------------------------------------------------------------------

PROCESS {
	# checking report name and apply default one if not specified
	if ($ReportName -eq "") {
		$TimeStamp = Get-Date -Format "yyyyMMdd.hhMMss"
		$InnerReportName = Join-Path -Path (Join-Path -Path $env:HOMEDRIVE -ChildPath $env:HOMEPATH) -ChildPath "MailboxFolderPermissionReport-$TimeStamp-$MailboxName.html"
		Write-Host -ForegroundColor Yellow ("Setting reportname to: $InnerReportName")
	}
	else {
		$InnerReportName = $ReportName
	}

	# get mailbox
	Write-Host -ForegroundColor Green ("processing mailbox: ") -NoNewline
	Write-Host -ForegroundColor Yellow ($MailboxName)
	$Mailbox = Get-Mailbox -Identity $MailboxName -WarningAction SilentlyContinue -ErrorAction SilentlyContinue
	
	# check if mailbox esists
	if ($Mailbox) {
		Write-Host -ForegroundColor Green ("  getting mailbox folders...")
		#getting folders within selected mailbox
		$MailboxFolders = Get-MailboxFolderStatistics -Identity $Mailbox.PrimarySmtpAddress.ToString() `
														-FolderScope All | `
							#Exclude system folders
							Where-Object {$_.FolderType -ne "RecoverableItemsPurges" -and `
											$_.FolderType -ne "RecoverableItemsVersions" -and `
											$_.FolderType -ne "RecoverableItemsDeletions" -and `
											$_.FolderType -ne "RecoverableItemsRoot" -and `
											$_.FolderType -ne "ServerFailures" -and `
											$_.FolderType -ne "LocalFailures" -and `
											$_.FolderType -ne "Conflicts" -and `
											$_.FolderType -ne "SyncIssues" -and `
											$_.FolderType -ne "Root" -and `
											$_.Name -ne "Calendar Logging"} | `
							Select-Object -Property Name, FolderPath, Permissions, FolderType

		# getting mailbox folder permissions	
		Write-Host -ForegroundColor Green ("  getting mailbox folderpermissions...")
		$MailboxFolders | ForEach-Object {
			$MBXFolderIdentity = $Mailbox.PrimarySmtpAddress.ToString()+":"+($_.FolderPath).Replace("/", "`\")
			$CurrentFolderType = $_.FolderType
			try {
				$AllPermissions = Get-MailboxFolderPermission -Identity $MBXFolderIdentity -WarningAction SilentlyContinue -ErrorAction SilentlyContinue
			}
			catch {}
	
			$runner = 0
			$FilteredPermissions = New-Object System.Collections.Generic.List[System.String]
			$AllPermissions | ForEach-Object {
				$addThis = $true
				if ($AllPermissions) {
					if ($HideDefaultPermissions) {
						if (($AllPermissions[$runner].AccessRights -eq "none" -and ($AllPermissions[$runner].User).ToString() -eq "Default") -or `
							($AllPermissions[$runner].AccessRights -eq "none" -and ($AllPermissions[$runner].User).ToString() -eq "Anonymous")) {
							$addThis = $false
						}
						if ($CurrentFolderType -eq "Calendar" -and $AllPermissions[$runner].AccessRights -eq "AvailabilityOnly" -and ($AllPermissions[$runner].User).ToString() -eq "Default") {
							$addThis = $false
						}
					}
					if ($HideMailboxOwner -and ($AllPermissions[$runner].User).ToString() -eq $Mailbox.DisplayName) {
						$addThis = $false
					}
					#Add permission entry...
					if ($addThis) {
						$FilteredPermissions.Add(($AllPermissions[$runner].User).ToString() + " :: " + ($AllPermissions[$runner].AccessRights))
					}
				}
				else {
					$FilteredPermissions.Add("NO PERMISSION ENTRY ON FOLDER")
				}
				$runner++
			}
			$_.FolderPath = $_.FolderPath.SubString(1)	
			$_.Permissions = $FilteredPermissions.ToArray()
		}
	
		# creating html report
		Write-Host -ForegroundColor Green "  creating html report..."
		$HTML = $null
		# mailbox leaf
		$HTML += ("<ul>")
		$HTML += ("<li><h1>Permission report for: "+$Mailbox.DisplayName+"</h1>("+$Mailbox.PrimarySmtpAddress+")")
		$global:oldlayer = 0
		$MailboxFolders | ForEach-Object {
			#Root Folder
			$layers = ($_.FolderPath.Split("/")).Count
			for ($layer=1; $layer -le $layers; $layer++) {
				if ($global:oldlayer -eq $layer -and $layer -eq $layers) {
					$HTML += ("</li>")
				}
				if ($global:oldlayer -lt $layer) {
					$HTML += ("<ul>")
				}
				elseif ($global:oldlayer -gt $layers) {
					while ($global:oldlayer -gt $layers) {
						$HTML += ("</li></ul>")
						$global:oldlayer--
					}
					if ($global:oldlayer -eq $layers) {
						$HTML += ("</li>")
					}
				}
				if ($layer -eq $layers) {
					$HTML += ("<li><b>"+$_.Name+"</b>")
					foreach ($entry in $_.Permissions) {
						$HTML += ("<br>"+$entry)
					}
					$global:oldlayer = $layer
				}
			}
		}
		if ($layer -gt $global:oldlayer) {
			$HTML += ("</li></ul>")
		}
		$HTML += "</li></ul></li></ul>"
		try {
			if ($AppendReport) {
				$HTML | Out-File -FilePath $InnerReportName -Encoding utf8 -Append
			}
			else {
				$HTML | Out-File -FilePath $InnerReportName -Encoding utf8
			}
		}
		catch {
			Write-Host -ForegroundColor Red "  ERROR while writing report to file: $InnerReportName"
		}
	}
	else {
		Write-Host -ForegroundColor Red ("  ERROR: Mailbox: $MailboxName could not be found!")
	}
}
END {
	#disconnect the exchange shell
	Handle-ExchangeShell -ExchangeServerName $ExchangeServer -Mode "disconnect"   
	Write-Host -ForegroundColor Green "... Done"
}
